/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.datacollector;

import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.validator.DynaValidatorForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.DateHelper;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * Display all adminsitration stuffs for the datacollector
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class ShowDatacollectorStatus extends Action {
    @Override
    public ActionForward execute(ActionMapping iActionMapping, ActionForm form,
            HttpServletRequest iRequest, HttpServletResponse iResponse) {
        DynaValidatorForm dcForm = (DynaValidatorForm) form;

        final Logger log = Logger.getLogger(ShowDatacollectorAdmin.class);

        /** deals with error messages * */
        AdminSession adminSession = AdminSession.getAdminSession(iRequest.getSession());
        InfoBean infoBean = adminSession.getInfoBean();

        try {
            if (CollectorManager.isCollectorInitialized()) {
                CollectorServer collector = CollectorManager.getCollector();
                dcForm.set("petalsHost", String.valueOf(collector.getHost()));
                dcForm.set("petalsPort", String.valueOf(collector.getPort()));
                dcForm.set("petalsUsername", String.valueOf(collector.getUser()));

                dcForm.set("dbPath", String.valueOf(collector.getHSQLDBPath()));
                dcForm.set("dbPort", String.valueOf(collector.getHSQLDBPort()));

                dcForm.set("startedDuration", DateHelper.formatDuration((new Date()).getTime()
                        - collector.getStartTime()));
                dcForm.set("messagesCount", String.valueOf(collector.getDefaultCollectorClient().getMonitoringService().getMessagesExchangesCount()));
                dcForm.set("messagesSnapshotCount", String.valueOf(collector.getDefaultCollectorClient().getMonitoringService().getMessagesExchangesSnapshotCount()));
                dcForm.set("serverVersion", collector.getHSQLDBServerVersion());
           }

        } catch (DataCollectorServerException e) {
            infoBean.setMessage("Error : " + StringHelper.formatExceptionMessage(e.getMessage()));
            infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
            log.error("Error : can't admin datacollector", e);
            return iActionMapping.findForward("failure");
        } catch (DataCollectorClientException e) {
            infoBean.setMessage("Error : " + StringHelper.formatExceptionMessage(e.getMessage()));
            infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
            log.error("Error : can't admin datacollector", e);
            return iActionMapping.findForward("failure");
        } catch (DataCollectorServerNotInitializedException e) {
            infoBean.setMessage("Error : " + StringHelper.formatExceptionMessage(e.getMessage()));
            infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
            log.error("Error : can't admin datacollector", e);
            return iActionMapping.findForward("failure");
        }

        return iActionMapping.findForward("success");

    }

}
