/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.datacollector;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.apache.struts.validator.DynaValidatorForm;
import org.hsqldb.DatabaseManager;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.Logon;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * Display all adminsitration stuffs for the datacollector
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class StartStopDatacollector extends Action {
    @Override
    public ActionForward execute(ActionMapping iActionMapping, ActionForm form,
            HttpServletRequest iRequest, HttpServletResponse iResponse) {

        final Logger log = Logger.getLogger(Logon.class);

        DynaActionForm dcForm = (DynaActionForm) form;
        String action = (String) dcForm.get("action");

        if (StringHelper.isNullOrEmpty(action)) {
            action = iRequest.getParameter("action");
        }
        AdminSession adminSession = AdminSession.getAdminSession(iRequest.getSession());

        /** deals with error messages * */
        InfoBean infoBean = adminSession.getInfoBean();

        if ("start".equalsIgnoreCase(action)) {
            String petalsUser = (String) dcForm.get("login");
            String petalsPort = (String) dcForm.get("port");
            String petalsHost = (String) dcForm.get("host");
            String petalsPwd = (String) dcForm.get("password");
            String dbPort = (String) dcForm.get("dbPort");

            try {
                if (!CollectorManager.isCollectorInitialized()) {
                    /**
                     * will start the embedded HSLQDB server if not already
                     * started
                     */
                    CollectorServer.startHSQLServer(Integer.parseInt(dbPort));
                    CollectorManager.initCollector(petalsHost, petalsPort, petalsUser, petalsPwd);
                    CollectorServer collector = CollectorManager.getCollector();
                    collector.checkHSQLDBServerIsRunning();
                }
            } catch (DataCollectorServerException e) {
                infoBean.setMessage("Error : "
                        + StringHelper.formatExceptionMessage(e.getMessage()));
                infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
                log.error("Error : login failure", e);
                CollectorManager.destroyCollector();
                return iActionMapping.findForward("failure");
            }
        } else {
            try {
                if (CollectorManager.isCollectorInitialized()) {
                    CollectorServer collector = CollectorManager.getCollector();
                    collector.stopQueuesListening();
                    collector.stopMessagesListening();
                    CollectorServer.stopHSQLServer();
                    DatabaseManager.getTimer().shutDown();
                    CollectorManager.destroyCollector();
                }
            } catch (DataCollectorServerException e) {
                infoBean.setMessage("Error : "
                        + StringHelper.formatExceptionMessage(e.getMessage()));
                infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
                log.error("Error : login failure", e);
                CollectorManager.destroyCollector();
                return iActionMapping.findForward("failure");
            }
        }

        return iActionMapping.findForward("success");

    }

}
